    <section class="others">
      <ul>
        <li><a href="{{ element.prev_story[1] }}">{{ element.prev_story[0] }}</a></li>
        <li><a href="{{ element.next_story[1] }}">{{ element.next_story[0] }}</a></li>
      </ul>
    </section>
