<img src="{{ element.uri }}" alt="{{ element.alt }}"{%if element.width %} width="{{ element.width }}"{% endif %}{% if element.height %} height="{{ element.height }}"{% endif %} class="biian_image-content" />